@@ -24,8 +24,10 @@ import select
 import socket
 import argparse
 from typing import *
+from os import environ
 from os.path import expanduser
 from socket import gethostname
+import traceback
 
 import slack
 
@@ -221,6 +223,7 @@ class Client:
 
         try:
             self.sl_client.send_file(dest, filename)
+            self._sendreply(0, 'Upload of %s completed' % filename)
         except Exception as e:
             print(e)
             self._sendreply(Replies.ERR_FILEERROR, 'Unable to send file')
@@ -344,7 +347,23 @@ class Client:
                 msg
             )
 
-    def slack_event(self, sl_ev):
+    def _joined(self, sl_ev: slack.Join) -> None:
+        """
+        Handle join events from slack, by sending a JOIN notification
+        to IRC.
+        """
+        user = self.sl_client.get_user(sl_ev.user)
+        if user.deleted:
+            return
+        channel = self.sl_client.get_channel(sl_ev.channel)
+        dest = b'#' + channel.name.encode('utf8')
+        if dest in self.parted_channels:
+            return
+        name = user.name.encode('utf8')
+        rname = user.real_name.replace(' ', '_').encode('utf8')
+        self.s.send(b':%s!%s@127.0.0.1 JOIN :%s\n' % (name, rname, dest))
+
+    def slack_event(self, sl_ev: slack.SlackEvent) -> None:
         if isinstance(sl_ev, slack.MessageDelete):
             self._message(sl_ev, '[deleted]')
         elif isinstance(sl_ev, slack.Message):
@@ -357,6 +376,8 @@ class Client:
         elif isinstance(sl_ev, slack.FileShared):
             f = self.sl_client.get_file(sl_ev)
             self._message(f.announce())
+        elif isinstance(sl_ev, slack.Join):
+            self._joined(sl_ev)
 
     def command(self, cmd: bytes) -> None:
         if b' ' in cmd:
@@ -419,11 +440,25 @@ def main() -> None:
                 'authentication, it\'s recommended to only allow local connections\n' \
                 'you can override this with -o')
 
-    sl_client = slack.Slack(args.tokenfile)
+    if 'PORT' in environ:
+        port = int(environ['PORT'])
+    else:
+        port = args.port
+
+    if 'TOKEN' in environ:
+        token = environ['TOKEN']
+    else:
+        try:
+            with open(args.tokenfile) as f:
+                token = f.readline().strip()
+        except (FileNotFoundError, PermissionError):
+            exit(f'Unable to open the token file {args.tokenfile}')
+
+    sl_client = slack.Slack(token)
     sl_events = sl_client.events_iter()
     serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     serversocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-    serversocket.bind((args.ip, args.port))
+    serversocket.bind((args.ip, port))
     serversocket.listen(1)
 
     poller = select.poll()
@@ -437,7 +472,7 @@ def main() -> None:
             poller.register(sl_client.fileno, select.POLLIN)
 
         # Main loop
-        timeout = 0.1
+        timeout = 7
         while True:
             s_event = poller.poll(timeout)  # type: List[Tuple[int,int]]
             sl_event = next(sl_events)
@@ -452,12 +487,10 @@ def main() -> None:
                     if i:
                         ircclient.command(i)
 
-            if sl_event:
+            while sl_event:
                 print(sl_event)
                 ircclient.slack_event(sl_event)
-                timeout = 0.1
-            else:
-                timeout = 7
+                sl_event = next(sl_events)
 
 if __name__ == '__main__':
     while True:
@@ -465,5 +498,6 @@ if __name__ == '__main__':
             main()
         except KeyboardInterrupt:
             break
-        except:
+        except Exception as e:
+            traceback.print_last()
             pass
